@@ -9,17 +9,20 @@ import six
 import json
 import yaml
 
-from django.core.exceptions import ValidationError
-
 from flex.context_managers import ErrorCollection
+from flex.exceptions import ValidationError
 from flex.serializers.core import (
     SwaggerSerializer,
     SchemaSerializer,
 )
 from flex.serializers.definitions import SwaggerDefinitionsSerializer
 from flex.utils import prettify_errors
-from flex.validation.request import generate_request_validator
-from flex.validation.response import generate_response_validator
+from flex.http import (
+    normalize_request,
+    normalize_response,
+)
+from flex.validation.request import validate_request
+from flex.validation.response import validate_response
 
 
 def load_source(source):
@@ -36,7 +39,6 @@ def load_source(source):
     """
     if isinstance(source, collections.Mapping):
         return source
-
     elif hasattr(source, 'read') and callable(source.read):
         raw_source = source.read()
     elif os.path.exists(os.path.expanduser(str(source))):
@@ -76,12 +78,11 @@ def parse(raw_schema):
         data=raw_schema,
     )
     if not definitions_serializer.is_valid():
-
         message = "Swagger definitions did not validate:\n\n"
         message += prettify_errors(definitions_serializer.errors)
         raise ValueError(message)
 
-    swagger_definitions = definitions_serializer.object
+    swagger_definitions = definitions_serializer.save()
 
     swagger_serializer = SwaggerSerializer(
         swagger_definitions,
@@ -94,7 +95,7 @@ def parse(raw_schema):
         message += prettify_errors(swagger_serializer.errors)
         raise ValueError(message)
 
-    return swagger_serializer.object
+    return swagger_serializer.save()
 
 
 def load(target):
@@ -116,22 +117,32 @@ def validate(schema, target=None, **kwargs):
     if not schema_serializer.is_valid():
         message = "JSON Schema did not validate:\n\n"
         message += prettify_errors(schema_serializer.errors)
-        raise ValueError(message)
+        raise ValidationError(message)
 
     if target is not None:
         validator = schema_serializer.save()
         validator(target)
 
 
-def validate_api_call(schema, request, response):
+def validate_api_call(schema, raw_request, raw_response):
+    request = normalize_request(raw_request)
+    response = normalize_response(raw_response)
+
     with ErrorCollection() as errors:
         try:
-            operation_definition = generate_request_validator(schema, inner=True)(request)
+            validate_request(
+                request=request,
+                schema=schema,
+            )
         except ValidationError as err:
-            errors['request'].append(err.messages)
+            errors['request'].add_error(err.messages or getattr(err, 'detail'))
             return
 
         try:
-            generate_response_validator(operation_definition, schema, inner=True)(response)
+            validate_response(
+                response=response,
+                request_method=request.method,
+                schema=schema,
+            )
         except ValidationError as err:
-            errors['response'].append(err.messages)
+            errors['response'].add_error(err.messages or getattr(err, 'detail'))
